@@ -280,6 +280,8 @@ class AnalyzeErrors(object):
     def __init__(self, filename="errors.txt", time_max_threshold=30*1000):
         self.filename = prepare_file(filename)
         self.time_max_threshold = time_max_threshold  # in ms
+        self.dot_symbols = ['i', 'j', '\cdot', '\div', '\\because',
+                            '\\therefore']
 
     def __repr__(self):
         return "AnalyzeErrors"
@@ -287,47 +289,11 @@ class AnalyzeErrors(object):
     def __str__(self):
         return "AnalyzeErrors"
 
-    def __call__(self, raw_datasets):
+    def _write_data(self, symbols, err_recs, nr_raw_datasets,
+                    total_error_count, percentages, time_max_list):
+        """Write all obtained data to a file."""
         write_file = open(self.filename, "a")
-
-        # Initialize variables
         s = ""
-        symbols = defaultdict(int)
-        wild_point_recs = 0
-        wild_points = 0
-        missing_stroke_recs = 0
-        recs_with_errors = 0
-        recs_with_single_dot = 0  # except symbols_with_dots
-        single_dots = 0
-        symbols_with_dots = ['i', 'j', '\cdot', '\div', '\\because',
-                             '\\therefore']
-        percentages = []
-        time_max_list = []  # List with symbols that are over the time maximum
-
-        for raw_dataset in raw_datasets:
-            symbols[raw_dataset['handwriting'].formula_in_latex] += 1
-            if raw_dataset['handwriting'].get_time() > self.time_max_threshold:
-                time_max_list.append(raw_dataset['handwriting'])
-            if raw_dataset['handwriting'].wild_point_count > 0:
-                wild_point_recs += 1
-                wild_points += raw_dataset['handwriting'].wild_point_count
-            missing_stroke_recs += raw_dataset['handwriting'].missing_stroke
-            if raw_dataset['handwriting'].wild_point_count > 0 or \
-               raw_dataset['handwriting'].missing_stroke:
-                recs_with_errors += 1
-            if raw_dataset['handwriting'].count_single_dots() > 0 and \
-               raw_dataset['formula_in_latex'] not in symbols_with_dots and \
-               "dots" not in raw_dataset['formula_in_latex']:
-                recs_with_single_dot += 1
-                old_area = raw_dataset['handwriting'].get_area()
-                tmp = [preprocessing.RemoveDots()]
-                raw_dataset['handwriting'].preprocessing(tmp)
-                new_area = raw_dataset['handwriting'].get_area()
-                percentage = float(new_area)/float(old_area)
-                if percentage < 1.0:
-                    percentages.append(percentage)
-            single_dots += raw_dataset['handwriting'].count_single_dots()
-
         for symbol, count in sorted(symbols.items(), key=lambda n: n[0]):
             if symbol in ['a', '0', 'A']:
                 s += "\n%s (%i), " % (symbol, count)
@@ -344,37 +310,86 @@ class AnalyzeErrors(object):
 
         # Show errors
         print("Recordings with wild points: %i (%0.2f%%)" %
-              (wild_point_recs,
-               float(wild_point_recs)/len(raw_datasets)*100),
+              (err_recs['wild_points'],
+               float(err_recs['wild_points'])/nr_raw_datasets*100),
               file=write_file)
-        print("wild points: %i" % wild_points, file=write_file)
-        print("Recordings with missing line: %i (%0.2f%%)" %
-              (missing_stroke_recs,
-               float(missing_stroke_recs)/len(raw_datasets)*100),
+        print("wild points: %i" % total_error_count['wild_points'],
+              file=write_file)
+        print("Recordings with missing stroke: %i (%0.2f%%)" %
+              (err_recs['missing_stroke'],
+               float(err_recs['missing_stroke'])/nr_raw_datasets*100),
               file=write_file)
         print("Recordings with errors: %i (%0.2f%%)" %
-              (recs_with_errors,
-               float(recs_with_errors)/len(raw_datasets)*100),
+              (err_recs['total'],
+               float(err_recs['total'])/nr_raw_datasets*100),
               file=write_file)
         print("Recordings with dots: %i (%0.2f%%)" %
-              (recs_with_single_dot,
-               float(recs_with_single_dot)/len(raw_datasets)*100),
+              (err_recs['single_dots'],
+               float(err_recs['single_dots'])/nr_raw_datasets*100),
               file=write_file)
-        print("dots: %i" % single_dots)
+        print("dots: %i" % total_error_count['single_dots'], file=write_file)
         print("size changing removal: %i (%0.2f%%)" %
               (len(percentages),
-               float(len(percentages))/len(raw_datasets)*100),
+               float(len(percentages))/nr_raw_datasets*100),
               file=write_file)
+        print("%i recordings took more than %i ms. That were: " %
+              (len(time_max_list), self.time_max_threshold),
+              file=write_file)
+        for recording in time_max_list:
+            print("* %ims: %s: %s" %
+                  (recording.get_time(),
+                   utils.get_readable_time(recording.get_time()),
+                   recording),
+                  file=write_file)
+        write_file.close()
+
+    def __call__(self, raw_datasets):
+        # Initialize variables
+        symbols = defaultdict(int)
+
+        # Count errornous recordings
+        err_recs = {'wild_points': 0, 'missing_stroke': 0,
+                    'single_dots': 0,  # except symbols_with_dots
+                    'total': 0}
+
+        # Count errors (one type of error might occur multiple times in
+        # a single recording)
+        total_error_count = {'wild_points': 0, 'single_dots': 0}
+
+        percentages = []
+
+        # List with recordings that are over the time maximum
+        time_max_list = []
+
+        for raw_dataset in raw_datasets:
+            recording = raw_dataset['handwriting']
+            symbols[recording.formula_in_latex] += 1
+            if recording.get_time() > self.time_max_threshold:
+                time_max_list.append(recording)
+            if recording.wild_point_count > 0:
+                err_recs['wild_points'] += 1
+                total_error_count['wild_points'] += recording.wild_point_count
+            err_recs['missing_stroke'] += recording.missing_stroke
+            if recording.wild_point_count > 0 or \
+               recording.missing_stroke:
+                err_recs['total'] += 1
+            if recording.count_single_dots() > 0 and \
+               raw_dataset['formula_in_latex'] not in self.dot_symbols and \
+               "dots" not in raw_dataset['formula_in_latex']:
+                err_recs['single_dots'] += 1
+                old_area = recording.get_area()
+                tmp = [preprocessing.RemoveDots()]
+                recording.preprocessing(tmp)
+                new_area = recording.get_area()
+                percentage = float(new_area)/float(old_area)
+                if percentage < 1.0:
+                    percentages.append(percentage)
+            total_error_count['single_dots'] += recording.count_single_dots()
 
         time_max_list = sorted(time_max_list,
                                key=lambda n: n.get_time(),
                                reverse=True)
-        print("%i recordings took more than %i ms. That were: " %
-              (len(time_max_list), self.time_max_threshold),
-              file=write_file)
-        for el in time_max_list:
-            print("* %ims: %s: %s" % (el.get_time(),
-                                      utils.get_readable_time(el.get_time()),
-                                      el),
-                  file=write_file)
-        write_file.close()
+
+        self._write_data(symbols, err_recs, len(raw_datasets),
+                         total_error_count, percentages,
+                         time_max_list)
